package dev.pfaff.altargui.glfw;

import dev.pfaff.glass.glfw.GLFWwindow;
import dev.pfaff.log4truth.Logger;

// TODO: fold into Application
public final class GLFWUIUtil {
	private final GLFWwindow window;
	private final DelayedInitGuiContext gui;
	private int framebufferWidth;
	private int framebufferHeight;
	private int windowWidth;
	private int windowHeight;
	private float contentScale;
	private float screenCoordinatesToVirtualPixelsScaleFactor;

	public GLFWUIUtil(GLFWwindow window, DelayedInitGuiContext gui) {
		this.window = window;
		this.gui = gui;
		assert window.equals(GLFWwindow.getCurrentContext(window.glfw()));
		var fbSize = window.getFramebufferSize();
		var winSize = window.getWindowSize();
		var contentScale = window.getWindowContentScale();
		this.framebufferWidth = fbSize.width();
		this.framebufferHeight = fbSize.height();
		this.windowWidth = winSize.width();
		this.windowHeight = winSize.height();
		this.contentScale = contentScale.min();
		this.updateUI();
	}

	public void framebufferSizeChanged(int width, int height) {
		assert window.equals(GLFWwindow.getCurrentContext(window.glfw()));
		this.framebufferWidth = width;
		this.framebufferHeight = height;
		this.updateUI();
	}

	public void contentScaleChanged(float x, float y) {
		assert window.equals(GLFWwindow.getCurrentContext(window.glfw()));
		this.contentScale = Math.min(x, y);
		this.updateUI();
	}

	public void windowSizeChanged(int width, int height) {
		assert window.equals(GLFWwindow.getCurrentContext(window.glfw()));
		this.windowWidth = width;
		this.windowHeight = height;
		this.recomputeVirtualScaleFactor();
	}

	private void updateUI() {
		assert window.equals(GLFWwindow.getCurrentContext(window.glfw()));
		Logger.log(() -> STR."updateUI: framebufferWidth=\{framebufferWidth}, framebufferHeight=\{framebufferHeight}, contentScale=\{contentScale}");
		this.recomputeVirtualScaleFactor();
		gui.onWindowResize(framebufferWidth, framebufferHeight, contentScale);
	}

	/**
	 * The scale factor necessary to convert screen coordinates (i.e. mouse position from GLFW callback) to "virtual
	 * pixels" used by Altar GUI.
	 */
	public float screenCoordinatesToVirtualPixelsScaleFactor() {
		return this.screenCoordinatesToVirtualPixelsScaleFactor;
	}

	private void recomputeVirtualScaleFactor() {
		this.screenCoordinatesToVirtualPixelsScaleFactor = (float) (framebufferWidth / (double) windowWidth / contentScale);
		Logger.log(() -> STR."screenCoordinatesToVirtualPixelsScaleFactor=\{screenCoordinatesToVirtualPixelsScaleFactor}");
	}
}
